<?php

declare(strict_types=1);

namespace Drupal\field_ipaddress_pgsql\Plugin\Field\FieldFormatter;

use Drupal\Core\Database\Connection;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'IP Address PostgreSQL cidr' formatter.
 *
 * @FieldFormatter(
 *   id = "ipaddress_cidr",
 *   label = @Translation("IP Address PostgreSQL cidr"),
 *   field_types = {
 *     "ipaddress_cidr",
 *   }
 * )
 */
final class IpAddressCidrFormatter extends FormatterBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs the plugin instance.
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    private readonly Connection $connection,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new self(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    $setting = [
      'abbrev' => FALSE,
      'broadcast' => FALSE,
      'family' => FALSE,
      'host' => FALSE,
      'hostmask' => FALSE,
      'masklen' => FALSE,
      'netmask' => FALSE,
      'network' => FALSE,
    ];
    return $setting + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);

    $form['abbrev'] = [
      '#type' => 'checkbox',
      '#title' => 'abbrev',
      '#description' => $this->t('Creates an abbreviated display format as text.'),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('abbrev'),
    ];

    $form['broadcast'] = [
      '#type' => 'checkbox',
      '#title' => 'broadcast',
      '#description' => $this->t("Computes the broadcast address for the address's network."),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('broadcast'),
    ];

    $form['family'] = [
      '#type' => 'checkbox',
      '#title' => 'family',
      '#description' => $this->t("Returns the address's family: 4 for IPv4, 6 for IPv6."),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('family'),
    ];

    $form['host'] = [
      '#type' => 'checkbox',
      '#title' => 'host',
      '#description' => $this->t('Returns the IP address as text, ignoring the netmask.'),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('host'),
    ];

    $form['hostmask'] = [
      '#type' => 'checkbox',
      '#title' => 'hostmask',
      '#description' => $this->t("Computes the host mask for the address's network."),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('hostmask'),
    ];

    $form['masklen'] = [
      '#type' => 'checkbox',
      '#title' => 'masklen',
      '#description' => $this->t('Returns the netmask length in bits.'),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('masklen'),
    ];

    $form['netmask'] = [
      '#type' => 'checkbox',
      '#title' => 'netmask',
      '#description' => $this->t("Computes the network mask for the address's network."),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('netmask'),
    ];

    $form['network'] = [
      '#type' => 'checkbox',
      '#title' => 'network',
      '#description' => $this->t('Returns the network part of the address, zeroing out whatever is to the right of the netmask.'),
      '#return_type' => TRUE,
      '#default_value' => $this->getSetting('network'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    return [
      $this->t('IP Address function to display additional information about the IP address.'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $element = [];

    foreach ($items as $delta => $item) {
      $element[$delta] = $this->viewValue($item->value);
    }

    return $element;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param string $ipAddressString
   *   The IP address in the "cidr" representation as a string.
   *
   * @return array
   *   The textual output generated as a render array.
   */
  protected function viewValue($ipAddressString) {

    // If a user has not selected any functions in the formatter settings,
    // then return just a cidr value.
    if (array_search(TRUE, $this->getSettings()) === FALSE) {
      return [
        '#theme' => 'field__ipaddress_pgsql',
        '#ip_address' => $ipAddressString,
        '#ip_address_details' => [],
      ];
    }

    // As a user has selected functions in the formatter settings, then we
    // create a query to the database and get the values of these functions.
    $select = 'SELECT ';

    if ($this->getSetting('abbrev')) {
      $select .= 'abbrev(cidr :ipAddressString) as "abbrev", ';
    }
    if ($this->getSetting('broadcast')) {
      $select .= 'broadcast(cidr :ipAddressString) as "broadcast", ';
    }
    if ($this->getSetting('family')) {
      $select .= 'family(cidr :ipAddressString) as "family", ';
    }
    if ($this->getSetting('host')) {
      $select .= 'host(cidr :ipAddressString) as "host", ';
    }
    if ($this->getSetting('hostmask')) {
      $select .= 'hostmask(cidr :ipAddressString) as "hostmask", ';
    }
    if ($this->getSetting('masklen')) {
      $select .= 'masklen(cidr :ipAddressString) as "masklen", ';
    }
    if ($this->getSetting('netmask')) {
      $select .= 'netmask(cidr :ipAddressString) as "netmask", ';
    }
    if ($this->getSetting('network')) {
      $select .= 'network(cidr :ipAddressString) as "network"';
    }

    $query = $this->connection->query($select, [
      ':ipAddressString' => $ipAddressString,
    ]);

    $result = $query->fetchAssoc();

    return [
      '#theme' => 'field__ipaddress_pgsql',
      '#ip_address' => $ipAddressString,
      '#ip_address_details' => $result,
    ];

  }

}
